{  1988-91, Bowers Development Corp. }
{ EventLoop.p }

Unit EventLoop;
Interface

Uses
{$ifc undefined Think_Pascal}
	Types,
	Quickdraw,
	Controls,
	Desk,
	Dialogs,
	DiskInit,
	Events,
	Fonts,
	Lists,
	Menus,
	OSEvents,
	TextEdit,
	ToolUtils,
{$endc}
	Editions,			{for InitEditionPack}
	EPPC,				{for kHighLevelEvent}
	GestaltEqu,			{for Gestalt, gestaltAppleEventsAttr, etc.}
	Traps;				{for _Unimplemented, _WaitNextEvent}

{----------}
Procedure Initialize;
Procedure DoUpdate;
Procedure DoDiskEvent;
Procedure DoApp4Event;
Procedure MainLoop;

{----------}
Implementation
Uses
{$ifc undefined Think_Pascal}
	StandardFile,		{for Miscellany}
{$endc}
	AppleEvents,
	AEvent,

	Globals,
	Cursors,
	DoScrap,
	WindowAids,
	Windowing,
	Dispatcher,
	MainMenu,
	Miscellany;

{$D+}
{$R+}
{$OV+}

{----------}
{$ifc undefined Think_Pascal}
Procedure _DataInit; External;
{$endc}

{----------}
Function TrapAvailable (tNumber:	integer;
						tType:		TrapType): boolean;
Begin
	TrapAvailable := (NGetTrapAddress (tNumber, tType)
					<> GetTrapAddress (_Unimplemented));
End; {TrapAvailable}

{----------}
Procedure SetStackSize (stackSize:	longint);
Begin
	SetApplLimit (Ptr (longint (@stackSize) - stackSize));
End; {SetStackSize}

{----------}
Procedure GetSysConfig;
var
	ignoreError:	OSErr;
	tempLong:		longint;	{for Gestalt}
	environs:		SysEnvRec;
begin
	with sysConfig do begin
		hasGestalt := (Gestalt (gestaltVersion, tempLong) = noErr);
		if hasGestalt then begin
			hasWNE := TrapAvailable (_WaitNextEvent, ToolTrap);
			ignoreError := Gestalt (gestaltQuickdrawVersion, tempLong);
			hasColorQD := (tempLong <> gestaltOriginalQD);
			hasAppleEvents := (Gestalt (gestaltAppleEventsAttr, tempLong) = noErr);
			hasEditionMgr := (Gestalt (gestaltEditionMgrAttr, tempLong) = noErr);
			if hasEditionMgr then begin
				if not CheckOS (InitEditionPack) then begin
					hasEditionMgr := false;		{unable to load Edition Manager package}
				end;
			end;
		end else begin
			ignoreError := SysEnvirons (curSysEnvVers, environs);
			if environs.machineType < 0 then begin
				hasWNE := false;
			end else begin
				hasWNE := TrapAvailable (_WaitNextEvent, ToolTrap);
			end;
			hasColorQD := environs.hasColorQD;
			hasAppleEvents := false;
			hasEditionMgr := false;
		end;
	end; {with}
end; {GetSysConfig}

{----------}
Procedure Initialize;
Begin
{$ifc undefined Think_Pascal}
	UnloadSeg (@_DataInit);
{$endc}

	SetStackSize (15000);
	MaxApplZone;
	MoreMasters;
	
	InitGraf (@ThePort);
	InitFonts;
	InitWindows;
	InitMenus;
	TEInit;
	InitDialogs (nil);
	
	FlushEvents (everyEvent - diskMask - app4Mask, 0);
	
	GetSysConfig;
	if sysConfig.hasAppleEvents then begin
		InitializeAE;
	end;
	inBackground := false;
	
	LoadMenus;
	LoadCursors;
	InitScrap; 
	
	InitGlobals;

	InitModelessDialogs;	
	InitTitles;		{initialize each menu title's module}
End; {Initialize}
 
{----------}
Procedure DoMouseDown;
Var
	whichWindow:	WindowPtr;
	whichPart:		integer;
Begin
	whichPart := FindWindow (curEvent.where, whichWindow);
	case whichPart of
		inDesk:
				{Do nothing};
		inMenuBar:
				DoMenu (MenuSelect (curEvent.where));
		inSysWindow:
				SystemClick (curEvent, whichWindow);
		inContent:
				DoContent (whichWindow);
		inDrag:
				DoDrag (whichWindow);
		inGrow:
				DoGrow (whichWindow);
		inGoAway:
				DoGoAway (whichWindow);
		inZoomIn:
				DoZoom (whichWindow, inZoomIn);
		inZoomOut:
				DoZoom (whichWindow, inZoomOut);
	end; {case}
End; {DoMouseDown}
 
{----------}
Procedure DoMouseUp;
Var
	whichWindow:	WindowPtr;
	whichPart:		integer;
Begin
	whichPart := FindWindow (curEvent.where, whichWindow);
	{save event time to check later for double click}
End; {DoMouseUp}
 
{----------}
Procedure DoKeystroke;
Var
	charCode:		integer;
	ch:				char;
Begin
	with curEvent do begin
		charCode := BitAnd (message, charCodeMask);
		ch := chr (charCode);
		
		if BitAnd (modifiers, cmdKey) <> 0 then begin
			if what <> autoKey then begin
				DoMenu (MenuKey (ch));
			end;
		end else begin
			if curWindow = nil then begin
				SysBeep (1);
			end else begin
				TypeInWindow (ch);
			end;
		end;
	end; {with}
End; {DoKeystroke}
 
{----------}
Procedure DoUpdate;
Var
	savePort:		GrafPtr;
	saveWindow:		WindowPtr;
	whichWindow:	WindowPtr;
Begin
	GetPort (savePort);
	whichWindow := WindowPtr (curEvent.message);
	SetPort (whichWindow);
	SaveWindow := curWindow;
	SetInfo (whichWindow);
	BeginUpdate (whichWindow);
		EraseRect (whichWindow^.portRect);
		DrawControls (whichWindow);
		UpdateContent;
	EndUpdate (whichWindow);
	SetInfo (saveWindow);	
	SetPort (savePort);
End; {DoUpdate}

{----------}
Procedure DoActivate;
Var
	activate:		boolean;
	whichWindow:	WindowPtr;
Begin
	whichWindow := WindowPtr (curEvent.message);
	SetPort (whichWindow);
	SetInfo (whichWindow);
	
	activate := odd (curEvent.modifiers);
	if activate then begin
		if cur^.text <> nil then begin
			TEActivate (cur^.text);
		end;
		ReadDeskScrap;
	end;
	
	HiliteScroll (cur^.vScroll, activate);
	HiliteScroll (cur^.hScroll, activate);
	ActivateContent (activate);

	if not activate then begin
		WriteDeskScrap;
		if cur^.text <> nil then begin
			TEDeActivate (cur^.text);
		end;
		SetInfo (nil);
	end;
End; {DoActivate}
 
{----------}
Procedure DoDiskEvent;
Const
	dlgTop			= 75;                                   
	dlgLeft			= 100;

Var
	ignore:			OSErr;
	where:			Point;
Begin
	if HiWord (curEvent.message) <> noErr then begin
		DILoad;
		SetPt (where, dlgLeft, dlgTop); 
		ignore := DIBadMount (where, curEvent.message);
		DIUnload;
	end;
End; {DoDiskEvent}
 
{----------}
Procedure DoApp4Event;
Const
	suspendResumeMsg	= 1;
	resumeMask			= 1;
	
Var
	resume:			boolean;
	front:			WindowPtr;
Begin
	curEvent.what := nullEvent;
	If BitShift (curEvent.message, -24) = suspendResumeMsg then begin
		resume := (BitAnd (curEvent.message, resumeMask) <> 0);
		if resume then begin
			ReadDeskScrap;
		end else begin
			WriteDeskScrap;
		end;
		inBackground := not resume;
		front := FrontWindow;
		if front <> nil then begin
			curEvent.what := activateEvt;
			curEvent.modifiers := LoWord (curEvent.message);
			curEvent.message := longint (front);
		end;
	end;
End; {DoApp4Event}
 
{----------}
Function GetSleep: longint;
Var
	sleep:			longint;
Begin
	sleep := GetCaretTime;	{if long time, some Inits won't run}
	if (not inBackground) & (FrontWindow <> nil) then begin
		if FrontWindow = curWindow then begin
			if cur^.text <> nil then begin
				with cur^.text^^ do begin
					if selStart = selEnd then begin
						sleep := GetCaretTime;
					end;
				end; {with}
			end;
		end else begin	{DA or modeless dialog}
			sleep := GetCaretTime;
		end;
	end;
	GetSleep := sleep;
End; {GetSleep}

{----------}
Function GetEvent: boolean;
Var
	gotEvent:		boolean;
Begin
	if sysConfig.hasWNE then begin
		gotEvent := WaitNextEvent (everyEvent, curEvent,
									GetSleep, cursorRgn);
	end else begin
		SystemTask;
		gotEvent := GetNextEvent (everyEvent, curEvent);
	end;
	GetEvent := gotEvent;
End; {GetEvent}
 
{----------}
Function DoEvent: boolean;
Var
	gotEvent:		boolean;
	whichDialog:	DialogPtr;
	itemHit:		integer;
Begin
	gotEvent := GetEvent;
	if curEvent.what = app4Evt then begin
	  { here so event won't get swallowed by IsDialogEvent }
	  { and so suspend/resume can be translated to activate }
		DoApp4Event;
	end;
	if IsDialogEvent (curEvent) then begin
		if (curEvent.what = activateEvt)
		or (curEvent.what = updateEvt) then begin
			whichDialog := DialogPtr (curEvent.message);
		end else begin
			whichDialog := FrontWindow;
		end;
		SetPort (whichDialog);
		if FilterModeless (whichDialog, curEvent, itemHit) then begin
			DoModelessItem (whichDialog, itemHit);
		end else if (curEvent.what = keyDown)
			& (BAnd (curEvent.modifiers, cmdKey) <> 0) 
		then begin
			DoKeystroke;	{=> calls MenuKey}
		end else if DialogSelect (curEvent, whichDialog, itemHit) then begin
			DoModelessItem (whichDialog, itemHit);
		end;
	end else if gotEvent then begin
		case curEvent.what of
			mouseDown:
					DoMouseDown;
			mouseUp:
					DoMouseUp;
			keyDown, autoKey:
					DoKeystroke;
			updateEvt:
					DoUpdate;
			activateEvt:
					DoActivate;
			diskEvt:
					DoDiskEvent;
			kHighLevelEvent: begin
					if sysConfig.hasAppleEvents then begin
						DoHighLevelEvent;
					end;
				end;
			otherwise
					DoIdle;
		end; {case}
	end;
	DoEvent := gotEvent;
End; {DoEvent}
 
{----------}
Procedure MainLoop;
Begin
	UpdateMenus;

	quittingTime := false;	
	repeat
		ShapeCursor;
		if cur^.text <> nil then begin
			TEIdle (cur^.text);
		end;
		if DoEvent then begin
			UpdateMenus;
		end;
	until quittingTime;
	
	WriteDeskScrap;
End; {MainLoop}

End. {EventLoop}
