//  1991, Bowers Development Corp.
// UAMLibraryM.h

//==========================================================
// Classes to support AppMaker user interface items: 		
//															
// Class		 (Subclass)		Description					
//----------------------------------------------------------
// TGrayLine	 (TView)		Gray line					
// TPalette		 (TCtlMgr)		Palette						
// TPictButton	 (TCtlMgr)		Picture button				
// TPictCheckBox (TCtlMgr)		Picture check box			
// TPictRadio 	 (TCtlMgr)		Picture radio button		
// TMultiPict 	 (TCtlMgr)		Multi-picture control		
// TSlider 		 (TCtlMgr)		Custom slider				
// TCustomCntl 	 (TCtlMgr)		Other custom control		
//==========================================================

#ifndef __UAMLibraryM__
#define __UAMLibraryM__

#ifndef __UMacApp__
	#include <UMacApp.h>
#endif

const short kPaletteProcID		=  		(200 * 16);			/* 3200 */

const short kPictButtonProcID	=		(201 * 16);			/* 3216 */
const short kPictCheckProcID	=		(201 * 16) + 1;		/* 3217 */	
const short kPictRadioProcID	=		(201 * 16) + 2;		/* 3218 */	
const short kMultiPictProcID	=		(201 * 16) + 3;		/* 3219 */	

const short kSliderProcID			=   (202 * 16);
const short kSliderGrayContinuousID	=	(202 * 16);			/* 3232 */
const short kSliderIndContinuousID	=	(202 * 16) + 1;		/* 3233 */
const short kSliderGrayDiscreteID	=	(202 * 16) + 2;		/* 3234 */
const short kSliderIndDiscreteID 	=	(202 * 16) + 3;		/* 3235 */

const short mMultiPictHit		= 101;
const short mPaletteHit			= 102;
const short mCustomHit			= 103;
const short mSliderHit			= 104;


/*----------*/
pascal void ActionProcForTSlider	(ControlHandle		aCMgrControl,
									 short				partCode);

/*----------*/
class TGrayLine : public TView {
  public:
  // no instance variables
  // methods:
	virtual pascal void Draw	(Rect		*area);		// override;
}; /* TGrayLine */

/*----------*/
struct PaletteTemplate {
	short		itsPictureID;
	short		itsNumberAcross;
	short		itsNumberDown;
	short		itsFrameSize;
};
typedef PaletteTemplate *PaletteTPtr;
	
/*----------*/
class TPalette : public TCtlMgr {
  public:
  // no instance variables
  // methods:
	virtual pascal void IPalette	(TView				*itsSuperView,
									 VPoint				*itsLocation, 
									 VPoint 			*itsSize,
									 SizeDeterminer		itsHSizeDet, 
									 SizeDeterminer		itsVSizeDet,
									 short				itsPictureID,
									 short				itsNumberAcross,
									 short				itsNumberDown,
									 short				itsFrameSize);
	virtual pascal void IRes 		(TDocument			*itsDocument,
									 TView				*itsSuperView,
									 Ptr				*itsParams); 		// Override;				
	virtual pascal void WRes		(ViewRsrcHndl		theResource,
									 Ptr				*itsParams); 		// Override;				
	virtual pascal void WriteRes	(ViewRsrcHndl		theResource,
									 Ptr				*itsParams); 		// Override;				
	virtual pascal void Fields		(pascal void (*DoToField)
										(StringPtr		fieldName, 
										 Ptr			fieldAddr, 
										 short			fieldType, 
										 void			*DoToField_StaticLink),
									 void	*DoToField_StaticLink);			// Override;
}; /* TPalette */

/*----------*/
struct PictControlTemplate {
	Str255		itsPictureIDs;
};
typedef PictControlTemplate *PictControlTPtr;

/*----------*/
class TPictButton : public TCtlMgr {
  public:
  // no instance variables
  // methods:
	virtual pascal void IPictButton		(TView				*itsSuperView,
										 VPoint				*itsLocation, 
										 VPoint 			*itsSize,
										 SizeDeterminer		itsHSizeDet, 
										 SizeDeterminer		itsVSizeDet,
										 StringPtr			itsPicts);	
	virtual pascal void IRes			(TDocument			*itsDocument,
										 TView				*itsSuperView,
										 Ptr				*itsParams); 	// Override;				
	virtual pascal void WRes			(ViewRsrcHndl		theResource,
										 Ptr				*itsParams); 	// Override;				
	virtual pascal void WriteRes		(ViewRsrcHndl		theResource,
										 Ptr				*itsParams); 	// Override;				
	virtual pascal void Fields			(pascal void (*DoToField)
											(StringPtr		fieldName, 
											 Ptr			fieldAddr, 
											 short			fieldType, 
											 void			*DoToField_StaticLink),
										 void	*DoToField_StaticLink);		// Override;
}; /* TPictButton */

/*----------*/
class TPictCheckBox	: public TCtlMgr {
  public:
  // no instance variables
  // methods:
	virtual pascal void IPictCheckBox	(TView				*itsSuperView,
										 VPoint				*itsLocation, 
										 VPoint 			*itsSize,
										 SizeDeterminer		itsHSizeDet, 
										 SizeDeterminer		itsVSizeDet,
										 StringPtr			itsPicts,
										 Boolean			isTurnedOn);	
	virtual pascal void IRes			(TDocument			*itsDocument,
										 TView				*itsSuperView,
										 Ptr				*itsParams); 	// Override;			
	virtual pascal void WRes			(ViewRsrcHndl		theResource,
										 Ptr				*itsParams); 	// Override;			
	virtual pascal void WriteRes		(ViewRsrcHndl		theResource,
										 Ptr				*itsParams); 	// Override;			
	virtual pascal void DoChoice		(TView				*origView,
										 short				itsChoice);		// Override;	
	virtual pascal Boolean IsOn			(void);								// Override;
	virtual pascal void SetState		(Boolean			state,
										 Boolean			redraw);		// Override;
	virtual pascal void Toggle			(Boolean			redraw);		// Override;
	virtual pascal void ToggleIf		(Boolean			matchState,
										 Boolean			redraw);		// Override;
	virtual pascal void Fields			(pascal void (*DoToField)
											(StringPtr		fieldName, 
											 Ptr			fieldAddr, 
											 short			fieldType, 
											 void			*DoToField_StaticLink),
										 void	*DoToField_StaticLink);		// Override;
}; /* TPictCheckBox */

/*----------*/
class TPictRadio : public TCtlMgr {
  public:
  // no instance variables
  // methods:
	virtual pascal void IPictRadio	(TView				*itsSuperView,
									 VPoint				*itsLocation, 
									 VPoint 			*itsSize,
									 SizeDeterminer		itsHSizeDet, 
									 SizeDeterminer		itsVSizeDet,
									 StringPtr			itsPicts,
									 Boolean			isTurnedOn);	
	virtual pascal void IRes		(TDocument			*itsDocument,
									 TView				*itsSuperView,
									 Ptr				*itsParams); 		// Override;			
	virtual pascal void WRes		(ViewRsrcHndl		theResource,
									 Ptr				*itsParams); 		// Override;			
	virtual pascal void WriteRes	(ViewRsrcHndl		theResource,
									 Ptr				*itsParams); 		// Override;			
	virtual pascal void DoChoice	(TView				*origView,
									 short				itsChoice);			// Override;	
	virtual pascal Boolean IsOn		(void);									// Override;
	virtual pascal void SetState	(Boolean			state,
									 Boolean			redraw);			// Override;
	virtual pascal void Toggle		(Boolean			redraw);			// Override;
	virtual pascal void ToggleIf	(Boolean			matchState,
									 Boolean			redraw);			// Override;
	virtual pascal void Fields		(pascal void (*DoToField)
										(StringPtr		fieldName, 
										 Ptr			fieldAddr, 
										 short			fieldType, 
										 void			*DoToField_StaticLink),
									 void	*DoToField_StaticLink);			// Override;
}; /* TPictRadio */

/*----------*/
class TMultiPict : public TCtlMgr {
  public:
  // no instance variables
  // methods:
	virtual pascal void IMultiPict	(TView				*itsSuperView,
									 VPoint				*itsLocation, 
									 VPoint 			*itsSize,
									 SizeDeterminer		itsHSizeDet, 
									 SizeDeterminer		itsVSizeDet,
									 StringPtr			itsPicts,
									 short				curValue);	
	virtual pascal void IRes		(TDocument			*itsDocument,
									 TView				*itsSuperView,
									 Ptr				*itsParams); 		// Override;			
	virtual pascal void WRes		(ViewRsrcHndl		theResource,
									 Ptr				*itsParams); 		// Override;			
	virtual pascal void WriteRes	(ViewRsrcHndl		theResource,
									 Ptr				*itsParams); 		// Override;			
	virtual pascal void DoChoice	(TView				*origView,
									 short				itsChoice);			// Override;	
	virtual pascal void Cycle		(Boolean			redraw);			// Override;
	virtual pascal void Fields		(pascal void (*DoToField)
										(StringPtr		fieldName, 
										 Ptr			fieldAddr, 
										 short			fieldType, 
										 void			*DoToField_StaticLink),
									 void	*DoToField_StaticLink);			// Override;
}; /* TMultiPict */

/*----------*/
struct SliderTemplate {
	short		itsVariantCode;
	Str255		itsPictureIDs;
};
typedef SliderTemplate *SliderTPtr;
	
/*----------*/
class TSlider : public TCtlMgr {
  public:
  // instance variables:
	VHSelect		fDirection;		// horizontal or vertical

  // methods:
	virtual pascal void ISlider		(TView				*itsSuperView,
									 VPoint				*itsLocation, 
									 VPoint 			*itsSize,
									 SizeDeterminer		itsHSizeDet, 
									 SizeDeterminer		itsVSizeDet,
									 VHSelect			itsDirection,
									 StringPtr			itsPicts,
									 short				itsProcID,
									 long				itsVal,
									 long				itsMin,
									 long				itsMax);	
	virtual pascal void IRes 		(TDocument			*itsDocument,
									 TView				*itsSuperView,
									 Ptr				*itsParams); 		// Override;			
	virtual pascal void WRes		(ViewRsrcHndl		theResource,
									 Ptr				*itsParams); 		// Override;			
	virtual pascal void WriteRes	(ViewRsrcHndl		theResource,
									 Ptr				*itsParams); 		// Override;			
	virtual pascal void DeltaValue	(VCoordinate		delta);				// Override;
	virtual pascal TCommand* DoMouseCommand	(Point			*theMouse, 
											 EventInfo 		*info, 
											 Point 			*hysteresis);	// Override;	
	virtual pascal void TrackSlider	(short			partCode);				// Override;
	virtual pascal void ActionProc	(short			partCode);				// Override;
	virtual pascal void Fields		(pascal void (*DoToField)
										(StringPtr		fieldName, 
										 Ptr			fieldAddr, 
										 short			fieldType, 
										 void			*DoToField_StaticLink),
									void	*DoToField_StaticLink);			// Override;
}; /* TSlider */

/*----------*/
struct CustomControlTemplate {
	short		itsCntlID;
};
typedef CustomControlTemplate *CustomControlTPtr;
	
/*----------*/
class TCustomCntl : public TCtlMgr {
  public:
  // instance variables:
	short		fCntlID;

  // methods:
	virtual pascal void ICustom		(TView				*itsSuperView,
									 VPoint				*itsLocation, 
									 VPoint 			*itsSize,
									 SizeDeterminer		itsHSizeDet, 
									 SizeDeterminer		itsVSizeDet,
									 short				itsCntlID);	
	virtual pascal void IRes		(TDocument			*itsDocument,
									 TView				*itsSuperView,
									 Ptr				*itsParams); 		// Override;			
	virtual pascal void WRes		(ViewRsrcHndl		theResource,
									 Ptr				*itsParams); 		// Override;		
	virtual pascal void WriteRes	(ViewRsrcHndl		theResource,
									 Ptr				*itsParams); 		// Override;		
	virtual pascal void Fields		(pascal void (*DoToField)
										(StringPtr		fieldName, 
										 Ptr			fieldAddr, 
										 short			fieldType, 
										 void			*DoToField_StaticLink),
									 void	*DoToField_StaticLink);			// Override;
}; /* TCustomCntl */

#endif /* UAMLibraryM.h */
